@@ -1,4 +1,4 @@
-use lrpeg::{lr1, Generator};
+use lrpeg::build_parser;
 use std::path::PathBuf;
 use std::{env, fs};
 
@@ -11,14 +11,10 @@ fn main() {
 
     let src = fs::read_to_string(&filename).expect("failed to read input");
 
-    let grammar = lr1::parse(&src);
-
-    let mut gen = Generator::new();
-
     let path = PathBuf::from(filename);
 
     println!(
         "{}",
-        gen.build(&grammar, path.file_stem().unwrap().to_str().unwrap())
+        build_parser(&src, path.file_stem().unwrap().to_str().unwrap())
     );
 }
