@@ -5,12 +5,21 @@ use std::path::Path;
 use unicode_xid::UnicodeXID;
 
 mod ast;
-pub mod lr1;
+mod check;
+mod parser;
 mod utils;
 
 use utils::{escape_char, escape_string};
 
-pub struct Generator {
+/// Generate a parser module for the peg described in source. The result is
+/// a rust module as String.
+pub fn build_parser(source: &str, modname: &str) -> String {
+    let mut gen = Generator::new();
+
+    gen.build(source, modname)
+}
+
+struct Generator {
     symbols: HashSet<String>,
     builtins: HashMap<ast::Expression, String>,
     terminals: HashMap<ast::Expression, String>,
@@ -23,7 +32,7 @@ impl Default for Generator {
 }
 
 impl Generator {
-    pub fn new() -> Self {
+    fn new() -> Self {
         Generator {
             symbols: HashSet::new(),
             builtins: HashMap::new(),
@@ -31,7 +40,11 @@ impl Generator {
         }
     }
 
-    pub fn build(&mut self, grammar: &ast::Grammar, modname: &str) -> String {
+    fn build(&mut self, source: &str, modname: &str) -> String {
+        let grammar = parser::parse(&source);
+
+        check::check_grammar(&grammar);
+
         // prepopulate builtins
         self.symbols.insert(String::from("Dot"));
         self.symbols.insert(String::from("WHITESPACE"));
@@ -211,8 +224,7 @@ impl Generator {
             return res.clone();
         }}
 
-        self.rule_memo.insert(key, Err(pos));
-    "#,
+        self.rule_memo.insert(key, Err(pos));"#,
             def.name, def.name,
         );
 
@@ -468,16 +480,10 @@ impl Generator {
             ast::Expression::MemoDefinition(rule_no) => {
                 format!(
                     r#"match self.rule_memo.get(&(pos, Rule::{})) {{
-                    Some(e) => {{
-                        let mut e = e.clone();
-                        if let Ok(res) = &mut e {{
-                            res.alternative = {};
-                        }}
-                        e
-                    }},
+                    Some(e) => e.clone(),
                     None => Err(pos),
                 }}"#,
-                    grammar.definitions[*rule_no].name, alt,
+                    grammar.definitions[*rule_no].name
                 )
             }
             ast::Expression::Optional(expr) => {
@@ -715,8 +721,8 @@ impl PEG {
                 }
 
                 res.push_str(&format!(
-                    r##"
-            regex_{}: Regex::new(r#"^{}"#).unwrap(),"##,
+                    r################"
+            regex_{}: Regex::new(r########"^{}"########).unwrap(),"################,
                     name, r,
                 ));
             }
@@ -886,7 +892,7 @@ impl PEG {
                         r#"
                 Terminal::{} => {{
                     if input[pos..].starts_with({}) {{
-                        Some(pos + {}.len())
+                        Some(pos + "{}".len())
                     }} else {{
                         None
                     }}
@@ -901,8 +907,8 @@ impl PEG {
                     res.push_str(&format!(
                         r#"
                 Terminal::{} => {{
-                    if input[pos..].starts_with({}) {{
-                        Some(pos + {}.len())
+                    if input[pos..].starts_with("{}") {{
+                        Some(pos + "{}".len())
                     }} else {{
                         None
                     }}
@@ -944,6 +950,10 @@ impl PEG {
     }
 }
 
+/// This function is supposed to called from a crate build.rs. You pass in the
+/// directory where your .peg files reside ("src/" for example), and for each of
+/// those, an .rs file with the generated parser will be created in the out
+/// directory. You generally want this to be in cargo's OUT_DIR.
 pub fn process_files(dir: &Path, out: &Path) {
     for entry in fs::read_dir(dir).expect("cannot read directory") {
         let entry = entry.expect("cannot read file");
@@ -953,15 +963,13 @@ pub fn process_files(dir: &Path, out: &Path) {
         } else if path.is_file() && path.extension() == Some(OsStr::new("peg")) {
             let src = fs::read_to_string(&path).expect("failed to read input");
 
-            let grammar = lr1::parse(&src);
-
             let mut gen = Generator::new();
 
-            let res = gen.build(&grammar, path.file_stem().unwrap().to_str().unwrap());
+            let res = gen.build(&src, path.file_stem().unwrap().to_str().unwrap());
 
             let new_path = out.join(path.with_extension("rs").file_name().unwrap());
 
-            fs::write(new_path, res).expect("failed to write result")
+            fs::write(new_path, res).expect("failed to write generated parser")
         }
     }
 }
